<?php
namespace FiloBlu\Refilo\Model;
use FiloBlu\Refilo\Api\CustomerGroupManagmentInterface;
use Magento\Customer\Api\CustomerRepositoryInterface;
use Magento\Customer\Api\Data\CustomerInterface;
use Psr\Log\LoggerInterface;
use Magento\Quote\Api\CartRepositoryInterface;

class CustomerGroupManagment implements CustomerGroupManagmentInterface
{
    /**
     * @var CustomerRepositoryInterface
     */
    private $customerRepository;
    /**
     * @var LoggerInterface
     */
    private $logger;
    /**
     * @var CartRepositoryInterface
     */
    private $cartRepository;

    /**
     * CustomerGroupManagment constructor.
     * @param CustomerRepositoryInterface $customerRepository
     * @param LoggerInterface $logger
     * @param CartRepositoryInterface $cartRepository
     */
    public function __construct(
        CustomerRepositoryInterface $customerRepository,
        LoggerInterface $logger,
        CartRepositoryInterface  $cartRepository){
        $this->customerRepository = $customerRepository;
        $this->logger = $logger;
        $this->cartRepository = $cartRepository;
    }

    /**
     * @param int $customerId
     * @param int $customerGroupId
     * @return CustomerInterface
     */
    public function updateCustomerGroup(int $customerId, int $customerGroupId)
    {
        try{
            $customer = $this->customerRepository->getById($customerId);
            $cart = $this->cartRepository->getActiveForCustomer($customerId,[$customer->getStoreId()]);
            $customer->setGroupId($customerGroupId);
            $this->customerRepository->save($customer);
            $cart->setCustomer($customer);
            $this->cartRepository->save($cart);
            return $customer;
        }catch (\Exception $e){
            $this->logger->critical($e->getMessage());
        }catch (\Throwable $t){
            $this->logger->critical($t->getMessage());
        }
    }
}
