<?php

namespace FiloBlu\Refilo\Model\Exporter;

use FiloBlu\Refilo\Model\Adapter\Mongo;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\App\ResourceConnection;
use Magento\Framework\Exception\NoSuchEntityException;
use Magento\Store\Model\Service\StoreConfigManager;
use Magento\Store\Model\StoreManagerInterface;

class Hreflang
{
    /**
     * @var ResourceConnection
     */
    private $resourceConnection;
    /**
     * @var Mongo
     */
    private $mongo;
    /**
     * @var ScopeConfigInterface
     */
    private $scopeConfig;
    /**
     * @var StoreManagerInterface
     */
    private $storeManager;
    /**
     * @var Hreflangs
     */
    protected $hreflangHelper;
    /**
     * @var StoreConfigManager
     */
    protected $storeConfigManager;

    /**
     * Menu constructor.
     * @param ResourceConnection $resourceConnection
     * @param ScopeConfigInterface $scopeConfig
     * @param Mongo $mongo
     * @param StoreManagerInterface $storeManager
     * @param StoreConfigManager $storeConfigManager
     */
    public function __construct(
        ResourceConnection $resourceConnection,
        ScopeConfigInterface $scopeConfig,
        Mongo $mongo,
        StoreManagerInterface $storeManager,
        StoreConfigManager $storeConfigManager
    ) {
        $this->resourceConnection = $resourceConnection;
        $this->scopeConfig = $scopeConfig;
        $this->mongo = $mongo;
        $this->storeManager = $storeManager;
        $this->storeConfigManager = $storeConfigManager;
    }

    /**
     * @throws \MongoDB\Driver\Exception\Exception
     */
    public function exportHrefLang()
    {
        $tableName = $this->resourceConnection->getTableName('url_rewrite');
        $bulkData = [];

        $query = "SELECT $tableName.*,store.code as store_code FROM $tableName INNER JOIN store on store.store_id = $tableName.store_id WHERE $tableName.store_id != 0";

        $result = $this->resourceConnection->getConnection()->fetchAll($query);
        foreach ($result as $item) {
            $url = new \stdClass();

            $url->_id = $item['url_rewrite_id'];
            $url->hreflang = $this->getHreflang($item['target_path'], $item['request_path'], $item['entity_type'], $result);

            $bulkData[] = $url;
        }

        if (!empty($bulkData)) {
            try {
                $this->mongo->update(
                    '_id',
                    $bulkData,
                    'urls',
                    false,
                    $this->scopeConfig->getValue('mongo/general/mongo_general_database')
                );
            } catch (\MongoDB\Driver\Exception\Exception $e) {
                echo $e->getMessage();
            }
        }
    }

    /**
     * @param $targetPath
     * @param $requestPath
     * @param $type
     * @param $result
     * @return array
     */
    private function getHreflang($targetPath, $requestPath, $type, $result)
    {
        $hrefArray = [];
        $isHomePage = $this->isHomePage($requestPath);
        foreach ($result as $url) {
            if ($isHomePage && $type === 'cms-page' && isset($url['store_code']) && isset($url['store_id']) && $url['store_id'] != 0) {
                $storeConfig = $this->storeConfigManager->getStoreConfigs([$url['store_code']]);
                $storeLocale = $storeConfig[0]->getLocale();
                $urlPath = '';
                try {
                    $store = $this->storeManager->getStore($url['store_id']);
                    $urlPath = $store->getBaseUrl();
                } catch (NoSuchEntityException $e) {
                    echo $e->getMessage();
                }
                $hrefArray[$storeLocale] = $urlPath;
            }
            if (!$isHomePage && $url['target_path'] === $targetPath && isset($url['store_code']) && isset($url['store_id']) && $url['store_id'] != 0) {
                $storeConfig = $this->storeConfigManager->getStoreConfigs([$url['store_code']]);
                $storeLocale = $storeConfig[0]->getLocale();
                $urlPath = '';
                try {
                    $store = $this->storeManager->getStore($url['store_id']);
                    $urlPath = $store->getBaseUrl() . $url['request_path'];
                } catch (NoSuchEntityException $e) {
                    echo $e->getMessage();
                }
                $hrefArray[$storeLocale] = $urlPath;
            }
        }
        return $hrefArray;
    }

    /**
     * @param $requestPath
     * @return bool
     */
    private function isHomePage($requestPath)
    {
        $homeIdentifier = $this->scopeConfig->getValue('web/default/cms_home_page');
        return $requestPath === $homeIdentifier;
    }
}
